import React, { useReducer, createContext, useMemo } from "react";
import Initiator from "@qltr/initiator";
const JobContext = createContext();
const ACTIONS = {
  CREATE: "c",
  UPDATE: "u",
  DELETE: "d",
  PIPELINE: "p",
};

const initialState = { jobs: [], pipelines: [] };

const reducer = (state, action) => {
  // Current Jobs
  const { jobs, pipelines } = state;
  var jobIndex;
  // Actions
  switch (action.type) {
    case ACTIONS.CREATE:
      jobs.push(action.job);
      return { ...state, jobs };

    case ACTIONS.UPDATE:
      jobIndex = jobs.findIndex(
        (j) => j.jobId === (action.job.jobId ?? action.jobId)
      );
      jobs[jobIndex] = { ...jobs[jobIndex], ...action.job };
      return { ...state, jobs };

    case ACTIONS.DELETE:
      jobIndex = jobs.findIndex((j) => j.jobId === action.jobId);
      jobs.splice(jobIndex, 1);
      return { ...state, jobs };

    case ACTIONS.PIPELINE:
      return { ...state, pipelines };
    default:
      return state;
  }
};

export const JobProvider = ({ children }) => {
  const [state, dispatch] = useReducer(reducer, initialState);
  const jobUpdate = (job, jobId) =>
    dispatch({ type: ACTIONS.UPDATE, jobId, job });
  const jobCreate = (job) =>
    dispatch({ type: ACTIONS.CREATE, job: { ...job, log: [] } });
  const jobDelete = (jobId) => dispatch({ type: ACTIONS.DELETE, jobId });
  const updatePipelines = (pipelines) =>
    dispatch({ type: ACTIONS.pipeline, pipelines });

  const context = {
    state,
    dispatch,
    jobUpdate,
    jobCreate,
    jobDelete,
    updatePipelines,
  };
  const contextValue = useMemo(() => context, [state, dispatch]);

  return (
    <JobContext.Provider value={contextValue}>{children}</JobContext.Provider>
  );
};
export default JobContext;
