import { useContext } from "react";
import { v4 as uuidv4 } from "uuid";
import JobContext from "@qltr/jobctx";
import Initiator from "@qltr/initiator";
import { useOneshotCore } from "./OneshotCore.jsx";
import { usePipelineCore } from "./PipelineCore.jsx";
import { useJobExtra } from "./JobExtra.jsx";
import { jobStatus, socketUrl } from "./job-config.js";

export function useJobCore() {
  const { state, jobUpdate, jobCreate, jobDelete } = useContext(JobContext);
  const { pipelineStart, pipelineCancel, pipelineDestroy } = usePipelineCore();
  const { oneshotStart, oneshotCancel, oneshotDestroy } = useOneshotCore();
  const jobExtra = useJobExtra();

  function retryAll(failing) {
    console.log("Would retry all failing tests!");
  }

  function jobCompose(builderCache) {
    if (builderCache.tree) return pipelineStart(builderCache);
    return oneshotStart(builderCache);
  }

  return {
    // Job Context
    state,
    // Job Core
    jobCompose,
    retryAll,
    // Oneshot
    oneshotStart,
    oneshotCancel,
    oneshotDestroy,
    // Pipeline
    pipelineCancel,
    pipelineDestroy,
    pipelineStart,
    // Job Extra
    ...jobExtra,
  };
}

export { jobStatus } from "./job-config.js";
