import { useQuery, useQueryClient } from "@tanstack/react-query";
import { silencedMock } from "@qltr/mocks/alerting-mock.js";
import { failingMock } from "@qltr/mocks/results-mock.js";
import { testsMock, mappingsMock } from "@qltr/mocks/catalog-mock.js";

const QUALITEER_URL = "/api";

const useMock = false;

const asMock = (data) => ({ data });

const fetchApi = (subPath) => async () =>
  fetch(`${QUALITEER_URL}${subPath}`).then((res) => res.json());

export const useCatalogTests = () =>
  useMock
    ? asMock(testsMock())
    : useQuery(["catalogTests"], fetchApi("/catalog/tests"));

export const usePipelineMappings = () =>
  useMock
    ? asMock(mappingsMock())
    : useQuery(["pipelineMappings"], fetchApi("/catalog/pipeline-mappings"));

export const useSilencedAlerts = () =>
  useMock
    ? asMock(silencedMock())
    : useQuery(["silenced"], fetchApi("/alerting/silenced"));

export const useCurrentlyFailing = () =>
  useMock
    ? asMock(failingMock())
    : useQuery(["failing"], fetchApi("/results/failing"));

export const useUpsertAlert = () => {
  const qc = useQueryClient();
  return async function upsertAlert(silenceRequest) {
    var { id, name, class: className, method, expires } = silenceRequest;
    const keepExpires = typeof expires === "string";
    name = name ? name : "*";
    className = className ? className : "*";
    method = method ? method : "*";
    const res = await fetch(`${QUALITEER_URL}/alerting/silence`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({
        id,
        name,
        class: className,
        method,
        expires,
        keepExpires,
      }),
    });
    qc.invalidateQueries(["silenced"]);
  };
};

export const useIgnoreResult = () => {
  const qc = useQueryClient();
  return async function ignoreResult(result) {
    const { id } = result;
    const res = await fetch(`${QUALITEER_URL}/results/ignore`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ id }),
    });
    qc.invalidateQueries(["failing"]);
  };
};
