import React, { useState, useContext } from "react";
import { useSilencedAlerts, useUpsertAlert } from "@qltr/queries";
import StoreContext from "@qltr/store";
import SilencedBox from "./SilencedBox.jsx";
import SilenceDialog, { useSilenceDialog } from "./SilenceDialog.jsx";

import SpeedDial from "@mui/material/SpeedDial";
import SpeedDialAction from "@mui/material/SpeedDialAction";
import SpeedDialIcon from "@mui/material/SpeedDialIcon";

import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";
import Typography from "@mui/material/Typography";
import Box from "@mui/material/Box";

export default function Alerting() {
  const [deleteOpen, setDeleteOpen] = useState(false);
  const { updateStore, silenceRequest } = useContext(StoreContext);
  const { isLoading, data: silenced } = useSilencedAlerts();
  const upsertAlert = useUpsertAlert();
  const [sdOpen, sdToggle, silence, setSilence, sdClose] = useSilenceDialog();
  const toggleDelete = () => setDeleteOpen(!deleteOpen);

  const handleDeleteClose = (confirmed) => () => {
    toggleDelete();
    if (!confirmed) return;
    const silenceReq = { ...silence };
    upsertAlert({ ...silenceReq, expires: null });
  };

  const handleClose = (silenceReq) => {
    if (!silenceReq) return;
    upsertAlert(silenceReq);
  };

  const quickAlertClick = () => {
    setSilence({});
    sdToggle();
  };

  const editSilence = (silence) => () => {
    sdToggle();
    setSilence(silence);
  };

  const removeSilence = (silence) => () => {
    setSilence(silence);
    toggleDelete();
  };

  return (
    <div className="alerting">
      {isLoading
        ? null
        : (silenced ?? []).map((v, i) => (
            <SilencedBox
              key={i}
              silenceEntry={v}
              editSilence={editSilence(v)}
              removeSilence={removeSilence(v)}
            />
          ))}

      {(silenced ?? []).length === 0 ? (
        <React.Fragment>
          <Box
            display="flex"
            alignItems="center"
            justifyContent="center"
            sx={{ flexFlow: "wrap" }}
          >
            <Typography variant="h4">No alerts silenced! </Typography>{" "}
          </Box>
          <Box
            display="flex"
            alignItems="center"
            justifyContent="center"
            sx={{ flexFlow: "wrap" }}
          >
            {" "}
            <Typography variant="h5">
              Click the '+' to create a new one!
            </Typography>
          </Box>
        </React.Fragment>
      ) : null}

      <Dialog
        open={deleteOpen}
        onClose={handleDeleteClose()}
        sx={{ "& .MuiDialog-paper": { width: "80%", maxHeight: 435 } }}
        maxWidth="xs"
      >
        <DialogTitle>Resume Alerting</DialogTitle>
        <DialogContent>Are you sure you want to resume alerting?</DialogContent>
        <DialogActions>
          <Button onClick={handleDeleteClose()}>Cancel</Button>
          <Button onClick={handleDeleteClose(true)} autoFocus>
            Remove
          </Button>
        </DialogActions>
      </Dialog>

      <SilenceDialog
        keepMounted
        open={sdOpen}
        onClose={sdClose}
        silence={silence}
      />

      <SpeedDial
        ariaLabel="Silence Alert"
        sx={{ position: "fixed", bottom: 16, right: 16 }}
        icon={<SpeedDialIcon />}
        onClick={quickAlertClick}
        open={false}
      />
    </div>
  );
}
