import { useState, useContext, useEffect } from "react";
import { useUpsertAlert } from "@qltr/queries";
import useMediaQuery from "@mui/material/useMediaQuery";
import { useTheme } from "@mui/material/styles";
import Button from "@mui/material/Button";
import DialogTitle from "@mui/material/DialogTitle";
import DialogContent from "@mui/material/DialogContent";
import DialogActions from "@mui/material/DialogActions";
import Dialog from "@mui/material/Dialog";
import TextField from "@mui/material/TextField";
import Toolbar from "@mui/material/Toolbar";

import SilenceSelector from "./SilenceSelector.jsx";
export function useSilenceDialog(isOpen = false) {
  const [open, setOpen] = useState(isOpen);
  const [silence, setSilence] = useState({});
  const upsertAlert = useUpsertAlert();
  const dialogToggle = () => setOpen(!open);
  const dialogClose = (confirmedSilence) => {
    setOpen(false);
    if (!confirmedSilence) return;
    upsertAlert(confirmedSilence);
    setSilence({});
  };
  return [open, dialogToggle, silence, setSilence, dialogClose];
}

export default function SilenceDialog(props) {
  const { silence, open, onClose } = props;
  const [duration, setDuration] = useState({ h: 0, m: 0 });
  const theme = useTheme();
  const fullScreen = useMediaQuery(theme.breakpoints.down("sm"));
  const [silenceEntry, setSilenceEntry] = useState(silence);

  const durationModified = duration.h !== 0 || duration.m !== 0;
  const silenceModified =
    silence.name !== silenceEntry.name ||
    silence.method !== silenceEntry.method ||
    silence.class !== silenceEntry.class;
  const modified =
    (Object.keys(silence).length > 0 && durationModified) || silenceModified;

  useEffect(() => {
    setSilenceEntry(silence);
    setDuration({ h: 0, m: 0 });
  }, [silence, open]);
  const handleCancel = () => onClose();

  const handleOk = () => onClose(silenceEntry);

  const updateSilence = (silenceType) => (e) => {
    const s = { ...silenceEntry };
    s[silenceType] = e.target.value;
    setSilenceEntry({ ...s, expires: duration });
  };

  const updateDuration = (dur) => {
    setDuration(dur);
    setSilenceEntry({ ...silenceEntry, expires: dur });
  };

  return (
    <Dialog
      sx={
        fullScreen
          ? {}
          : { "& .MuiDialog-paper": { width: "80%", maxHeight: 525 } }
      }
      maxWidth="xs"
      open={open}
      fullScreen={fullScreen}
    >
      <Toolbar sx={{ display: { sm: "none" } }} />
      <DialogTitle>Silence Alert</DialogTitle>
      <DialogContent>
        <TextField
          fullWidth
          label="Test Name"
          variant="standard"
          defaultValue={silence.name ?? ""}
          onChange={updateSilence("name")}
          margin="normal"
        />
        <TextField
          fullWidth
          label="Test Method"
          variant="standard"
          defaultValue={silence.method ?? ""}
          onChange={updateSilence("method")}
          margin="normal"
        />
        <TextField
          fullWidth
          label="Test Class"
          variant="standard"
          defaultValue={silence.class ?? ""}
          onChange={updateSilence("class")}
          margin="normal"
        />
        <SilenceSelector
          formerExpires={silence.expires}
          duration={duration}
          setDuration={updateDuration}
        />
      </DialogContent>
      <DialogActions>
        <Button autoFocus onClick={handleCancel}>
          Cancel
        </Button>
        <Button onClick={handleOk} disabled={!modified}>
          Ok
        </Button>
      </DialogActions>
    </Dialog>
  );
}
