import { useState, useContext } from "react";
import { useCurrentlyFailing, useSilencedAlerts } from "@qltr/queries";
import JobContext from "@qltr/jobctx";
import SilenceDialog, { useSilenceDialog } from "../alerting/SilenceDialog.jsx";
import FailingBox from "./FailingBox.jsx";
import QuickSilence, { useQuickSilence } from "./QuickSilence.jsx";
import FailingRetry from "./FailingRetry.jsx";

// MaterialUI
import Box from "@mui/material/Box";
import Typography from "@mui/material/Typography";

export default function Failing() {
  const { state: jobState } = useContext(JobContext);
  const { isLoading: failsLoading, data: failing } = useCurrentlyFailing();
  const { isLoading: silencesLoading, data: silences } = useSilencedAlerts();
  const [qsAnchor, qsTest, openQs, closeQs] = useQuickSilence();
  const [sdOpen, sdToggle, silence, setSilence, sdClose] = useSilenceDialog();

  const joinSilence = (result) => {
    if (!silences) return;
    const silence = silences.find((s) => s.name === result.name);
    if (silence) result.silence = silence;
  };

  const joinJob = (result) => {
    if (result.isPipeline) return;
    const job = jobState.jobs.find(
      (j) => !j.isPipeline && j.builderCache.testNames.includes(result.name)
    );
    if (job) result.job = job;
  };

  const joinPipeline = (result) => {
    if (!result.isPipeline) return;
    const pipeline = jobState.pipelines.find((p) =>
      p.selectedBranches.find((b) => b.name === result.name)
    );
    if (!pipeline) return;
    const pipelineJob = jobState.jobs.find(
      (j) =>
        j.isPipeline &&
        j.pipelineId === pipeline.id &&
        j.branchId === result.name
    );
    if (!pipelineJob) result.pipeline = pipeline;
    else result.job = pipelineJob;
  };

  const resultsCompiled = () => {
    if (failsLoading) return [];
    const compiled = [];
    for (var r of failing) {
      const rc = { ...r };
      joinSilence(rc);
      // Find associated job and skip pipeline
      joinJob(rc);
      // Find associated pipeline
      joinPipeline(rc);
      compiled.push(rc);
    }
    return compiled;
  };

  const alertClick = (result) => (e) => {
    if (e) e.preventDefault();
    if (e && e.type === "contextmenu" && result.silence)
      return openQs(e, result);
    const { name, class: className, method } = result;
    const testInfo = { name, class: className, method };
    const silenceInfo = result.silence ?? testInfo;
    sdToggle();
    setSilence(silenceInfo);
  };

  return (
    <div className="failing">
      <SilenceDialog
        keepMounted
        open={sdOpen}
        onClose={sdClose}
        silence={silence}
      />
      <QuickSilence
        anchorEl={qsAnchor}
        handleClose={closeQs}
        test={qsTest}
        editSilence={alertClick(qsTest)}
      />
      <FailingRetry failing={failsLoading ? [] : failing} />
      {!failsLoading && failing.length === 0 && (
        <Box display="flex" alignItems="center" justifyContent="center">
          <Typography variant="h4">No tests failing!</Typography>
        </Box>
      )}
      {!failsLoading &&
        resultsCompiled().map((v, i) => (
          <FailingBox key={i} failingTest={v} alertClick={alertClick(v)} />
        ))}
    </div>
  );
}
