// React
import React, { useState, useContext } from "react";
import StoreContext from "@qltr/store";
import { useJobCore } from "@qltr/jobcore";

// Components
import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";
import SpeedDial from "@mui/material/SpeedDial";

// Icons
import ReplayIcon from "@mui/icons-material/Replay";

export default function FailingRetry(props) {
  const { failing } = props;
  const { state: jobState, retryAll, toJob } = useJobCore();
  const { state: store } = useContext(StoreContext);
  const [open, setOpen] = useState(false);
  const toggleOpen = () => setOpen(!open);
  const dialogClose = (confirmed) => () => {
    toggleOpen();
    if (!confirmed) return;
    const jobId = retryAll(failing);
    if (!store.focusJob) return;
    toJob(jobId);
  };
  if (!failing || failing.length === 0) return;
  return (
    <React.Fragment>
      <Dialog
        open={open}
        onClose={dialogClose()}
        sx={{ "& .MuiDialog-paper": { width: "80%", maxHeight: 435 } }}
        maxWidth="xs"
      >
        <DialogTitle>Retry all failing tests?</DialogTitle>
        <DialogContent>
          <DialogContentText>
            This will create x jobs and run y tests
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={dialogClose()}>Cancel</Button>
          <Button onClick={dialogClose(true)} autoFocus>
            Yes
          </Button>
        </DialogActions>
      </Dialog>
      <SpeedDial
        ariaLabel="Retry All"
        sx={{ position: "fixed", bottom: 16, right: 16 }}
        icon={<ReplayIcon />}
        onClick={toggleOpen}
        open={false}
      />
    </React.Fragment>
  );
}
